/*
 * IBM Confidential
 * OCO Source Materials
 *  Copyright IBM 2008
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.j2ca.vta.inbound;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.logging.Level;

import javax.resource.ResourceException;
import javax.resource.spi.CommException;
import javax.transaction.xa.Xid;

import com.ibm.eismocker.rmidriver.DataMap;
import com.ibm.eismocker.rmidriver.EisEvent;
import com.ibm.eismocker.rmidriver.IDataObjectStore;
import com.ibm.eismocker.rmidriver.RMIObjectClient;
import com.ibm.j2ca.extension.eventmanagement.EventStoreWithXid;
import com.ibm.j2ca.extension.eventmanagement.XidImpl;
import com.ibm.j2ca.extension.logging.LogLevel;
import com.ibm.j2ca.extension.logging.LogUtilConstants;
import com.ibm.j2ca.extension.logging.LogUtils;
import com.ibm.j2ca.vta.VTAConstants;
import com.ibm.j2ca.vta.VTAResourceAdapter;
import com.ibm.j2ca.vta.VTAStructuredRecord;

/**
 * This class represents an EventStore from which events are retrieved during inbound communication
 */
public class VTAEventStoreWithXid implements EventStoreWithXid {
	private LogUtils logUtils;
	
	//added by VTADev team
	private RMIObjectClient rimSession = null;
	private IDataObjectStore dataObjectStore = null;

	
	/**
     * Create an eventStore instance.
     *
     * @param 	activationSpec 	inbound properties
     * @param 	resourceAdapter global properties and log/trace.
     * @throws 					ResourceException
     */
	public VTAEventStoreWithXid(VTAActivationSpecWithXid activationSpec, VTAResourceAdapter resourceAdapter) throws ResourceException {
		this.logUtils = resourceAdapter.getLogUtils();
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAConstants.VTAEVENTSTOREXID, "VTAEventStoreWithXid");

		String host = activationSpec.getHostname();
		int port = Integer.parseInt(activationSpec.getPortnumber());
		// log the properties
		if (logUtils.isTraceEnabled(Level.FINE)) {
			logUtils.trace(Level.FINEST, VTAConstants.VTAEVENTSTOREXID, "VTAEventStoreWithXid", "The host name is " + host);
			logUtils.trace(Level.FINEST, VTAConstants.VTAEVENTSTOREXID, "VTAEventStoreWithXid", "The port is " + port);
		}
		try {
			rimSession = new RMIObjectClient(host, port);
			dataObjectStore = rimSession.getDataObjectStore();
		} catch (RemoteException re) {
			logUtils.log(LogLevel.FATAL, LogUtilConstants.ADAPTER_RBUNDLE, VTAConstants.VTAEVENTSTOREXID, "VTAEventStoreWithXid", "6801");//$NON-NLS-1$			
			throw new CommException("The connect call has failed. The polling of events from the event store cannot be done.", re);
		}

		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAConstants.VTAEVENTSTOREXID, "VTAEventStoreWithXid");
	}

	/**
	 * For this event, get the complete Object that it represents.
	 *
	 * Gets the object for event for the specified event during polling. The object consist of the bytes for a bo in the event file specified by the
     * event. The base classes call the getRecordForEvent method on the EventStore during polling.
	 *
	 * @param 	event
	 * @return
	 * @throws 			CommException
	 * @throws 			ResourceException
	 * @see 			com.ibm.j2ca.extension.eventmanagement.EventStore#getObjectForEvent(com.ibm.j2ca.extension.eventmanagement.Event)
	 */
	public javax.resource.cci.Record getRecordForEvent(com.ibm.j2ca.extension.eventmanagement.Event event)
			throws javax.resource.ResourceException,
			javax.resource.spi.CommException {
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAConstants.VTAEVENTSTOREXID, "getRecordForEvent");

		VTAStructuredRecord record = new VTAStructuredRecord();
		try {
			String objectName = event.getEventType();
			String opName = ((VTAEvent) event).getOperationName();
			DataMap dataMap = (DataMap) dataObjectStore.getDataMapForEvent(event.getEventId());

			record.setRecordName(objectName);
			record.setOperationName(opName);
			record.setEISRepresentation(dataMap);
			record.setLogUtils(logUtils);
		}catch (RemoteException re) {
			logUtils.log(LogLevel.FATAL, LogUtilConstants.ADAPTER_RBUNDLE, VTAConstants.VTAEVENTSTOREXID, "getRecordForEvent", "6805", new Object[] { event.getEventId() });//$NON-NLS-1$			
			throw new CommException("The corresponding data could not be retrieved for an event.", re);			
		}catch (Exception e) {
			logUtils.log(LogLevel.FATAL, LogUtilConstants.ADAPTER_RBUNDLE, VTAConstants.VTAEVENTSTOREXID, "getRecordForEvent", "6805", new Object[] { event.getEventId() });//$NON-NLS-1$			
			throw new ResourceException("The corresponding data could not be retrieved for an event.", e);
		}

		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAConstants.VTAEVENTSTOREXID, "getRecordForEvent");
		return record;
	}

	/** 
	 * This method should store the XID in the event table for the specified event.  
	 *
	 * @param 	event
	 * @param 	xid
	 * @throws 			CommException
	 * @throws 			ResourceException
	 * @see 			com.ibm.j2ca.extension.eventmanagement.EventStoreWithXid#setEventTransactionID(com.ibm.j2ca.extension.eventmanagement.Event, com.ibm.j2ca.extension.eventmanagement.XidImpl)
	 */
	public void setEventTransactionID(
			com.ibm.j2ca.extension.eventmanagement.Event event,
			com.ibm.j2ca.extension.eventmanagement.XidImpl xid)
			throws javax.resource.ResourceException,
			javax.resource.spi.CommException {
	}

	/**
	 * This method should return an array of XIDs for events that have an XID, but are not yet marked processed.
	 * 
	 * @throws 	CommException
	 * @throws 	ResourceException
	 * @see 	com.ibm.j2ca.extension.eventmanagement.EventStoreWithXid#getPendingTransactions()
	 */
	public javax.transaction.xa.Xid[] getPendingTransactions()
			throws javax.resource.ResourceException,
			javax.resource.spi.CommException {
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAConstants.VTAEVENTSTOREXID, "getPendingTransactions");

		if(dataObjectStore==null){
			throw new ResourceException("The connection to the EIS Mocker is invalid.");			
		}
		ArrayList xids = new ArrayList();
		ArrayList raEvents = null;
		try {
			raEvents = dataObjectStore.getEvents();
		} catch (RemoteException re) {
			logUtils.log(LogLevel.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, VTAConstants.VTAEVENTSTOREXID, "getPendingTransactions", "6802");//$NON-NLS-1$
			//throw CommException
			//throw new ResourceException("The polling of events from the event store cannot be done.", e);
			throw new CommException("The polling of events from the event store cannot be done.", re);
		}
		for (int iev = 0; iev < raEvents.size(); iev++) {
			EisEvent metaData = (EisEvent) raEvents.get(iev);
			String xidStr = metaData.getXid();
			if (xidStr != null && !xidStr.equals("") && !xidStr.equals("0")) {
				Xid xid = new XidImpl(xidStr);
				xids.add(xid);
			}
		}
		int size = xids.size();
		Xid xidArry[] = new XidImpl[size];
		for (int ixid  = 0; ixid < size; ixid++) {
			xidArry[ixid] = (Xid) xids.get(ixid);
		}

		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAConstants.VTAEVENTSTOREXID, "getPendingTransactions");
		return xidArry;
	}

	/** 
     * This method should return the event object given a particular XID.  
     *
     * @param 	xid
     * @throws 	CommException
	 * @throws 	ResourceException
	 * @see 	com.ibm.j2ca.extension.eventmanagement.EventStoreWithXid#getEventForXid(com.ibm.j2ca.extension.eventmanagement.XidImpl)
	 */
	public com.ibm.j2ca.extension.eventmanagement.Event getEventForXid(
			com.ibm.j2ca.extension.eventmanagement.XidImpl xid)
			throws javax.resource.ResourceException,
			javax.resource.spi.CommException {
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAConstants.VTAEVENTSTOREXID, "getEventForXid");

		VTAEvent event = new VTAEvent();
		ArrayList raEvents = null;
		boolean found = false;
		try {
			raEvents = dataObjectStore.getEvents();
			for(int idx=0; idx<raEvents.size(); idx++){
				EisEvent metaData= (EisEvent)raEvents.get(idx);
				if(metaData.getXid().equals(xid.toString())){
					event.setEventId(metaData.getEventId());
					event.setEventType(metaData.getEventType());
					event.setEventStatus(metaData.getStatus());
					event.setOperationName(metaData.getOperation());
					found=true;
					break;
				}
			}
		} catch (RemoteException re) {
			logUtils.log(LogLevel.FATAL, LogUtilConstants.ADAPTER_RBUNDLE, VTAConstants.VTAEVENTSTOREXID, "getEventForXid", "6806", new Object[] {xid.toString()});//$NON-NLS-1$			
			throw new CommException("The corresponding event could not be retrieved for an xid.", re);
		}
		if(!found){
			logUtils.log(LogLevel.FATAL, LogUtilConstants.ADAPTER_RBUNDLE, VTAConstants.VTAEVENTSTOREXID, "getEventForXid", "6806", new Object[] {xid.toString()});//$NON-NLS-1$			
			throw new ResourceException("The corresponding event could not be retrieved for an xid.");			
		}

		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAConstants.VTAEVENTSTOREXID, "getEventForXid");
		return event;
	}

	/**
	 * This method should close the EIS connection and cleanup all allocated resources associated with this object.
	 *
	 * @throws 	CommException
	 * @throws 	ResourceException
	 * @see		com.ibm.j2ca.extension.eventmanagement.EventStoreWithXid#close()
	 */
	public void close() throws javax.resource.ResourceException,
			javax.resource.spi.CommException {
	}

	/**
	 * This method return true if event filtering is support and false otherwise.
	 *  
	 * @return  
	 * @see 	com.ibm.j2ca.extension.eventmanagement.internal.BaseEventStore#implementsFiltering()
	 */
	public boolean implementsFiltering() {
		return false;
	}

	/**
	 * This method should return an arrayList containing Event objects of the 
	 * specified status and types.  If the typeFilter is null, the adapter 
	 * should assume that all events are desired.  Return the events in the same
	 * order that they appeared in the event table.  The adapter should return 
	 * up to pollQuantity of either pending or in-progress events. 
	 * 
	 * @param	pollQuantity  	the number of events to poll
	 * @param  	status  		the status value we want
	 * @param  	eventTypeFilter event types.
	 * @return 					an ArrayList of events
	 * @throws 				    CommException
	 * @throws 					ResourceException
	 * @see 					com.ibm.j2ca.extension.eventmanagement.EventStore#getEvents(int, int, java.lang.String[])
	 */
	public java.util.ArrayList getEvents(int pollQuantity, int status,
			java.lang.String[] eventTypeFilter) throws javax.resource.ResourceException,
			javax.resource.spi.CommException {
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAConstants.VTAEVENTSTOREXID, "getEvents");

		ArrayList events = new ArrayList();
		if (pollQuantity == 0)
			return events;		
		try {
			ArrayList raEvents = dataObjectStore.getEvents();
			for (int i = 0; i < raEvents.size(); i++) {
				EisEvent metaData = (EisEvent) raEvents.get(i);

				VTAEvent event = new VTAEvent();
				event.setEventId(metaData.getEventId());
				event.setEventType(metaData.getEventType());
				event.setEventStatus(metaData.getStatus());
				event.setOperationName(metaData.getOperation());
				if (metaData.getStatus()==status){
					events.add(event);
					--pollQuantity;
					if (pollQuantity == 0)
						break;					
				}
			}
		} catch (RemoteException re) {
			logUtils.log(LogLevel.SEVERE, LogUtilConstants.ADAPTER_RBUNDLE, VTAConstants.VTAEVENTSTOREXID, "getEvents", "6802");//$NON-NLS-1$			
			throw new CommException("The polling of events from the event store cannot be done.", re);
		}
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAConstants.VTAEVENTSTOREXID, "getEvents");
		return events;
	}

	/**
	 * This method should delete the specified event from the event table
	 *
	 * @param 	event 	the event to be deleted
	 * @throws 			CommException
	 * @throws 			ResourceException
	 * @see 			com.ibm.j2ca.extension.eventmanagement.EventStore#deleteEvent(com.ibm.j2ca.extension.eventmanagement.Event)
	 */
	public void deleteEvent(com.ibm.j2ca.extension.eventmanagement.Event event)
			throws javax.resource.ResourceException,
			javax.resource.spi.CommException {
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAConstants.VTAEVENTSTOREXID, "deleteEvent");

		try {
			dataObjectStore.deleteEvent(event.getEventId());
		} catch (RemoteException re) {
			logUtils.log(LogLevel.FATAL, LogUtilConstants.ADAPTER_RBUNDLE, VTAConstants.VTAEVENTSTOREXID, "deleteEvent", "6804");//$NON-NLS-1$			
			throw new CommException("The adapter could not delete the event.", re);
		}

		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAConstants.VTAEVENTSTOREXID, "deleteEvent");
	}

	/**
	 * This method should update the status of the specified event with the new status.
	 * 
	 * @param	event 		the event to update
	 * @param 	newStatus   the new integer status of the event     
	 * @throws 				CommException
	 * @throws 				ResourceException
	 * @see 				com.ibm.j2ca.extension.eventmanagement.EventStore#updateEventStatus(com.ibm.j2ca.extension.eventmanagement.Event, int)
	 */
	public void updateEventStatus(
			com.ibm.j2ca.extension.eventmanagement.Event event, int newStatus)
			throws javax.resource.ResourceException,
			javax.resource.spi.CommException {
		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodEntrance(VTAConstants.VTAEVENTSTOREXID, "updateEventStatus");
		try {
			dataObjectStore.updateEventStatus(event.getEventId(), newStatus);
		} catch (RemoteException re) {
			logUtils.log(LogLevel.FATAL, LogUtilConstants.ADAPTER_RBUNDLE, VTAConstants.VTAEVENTSTOREXID, "updateEventStatus", "6803", new Object[] { newStatus+"" });//$NON-NLS-1$			
			throw new CommException("The adapter could not update the event status.", re);
		}

		if (logUtils.isTraceEnabled(Level.FINE))
			logUtils.traceMethodExit(VTAConstants.VTAEVENTSTOREXID, "updateEventStatus");
	}

	/** 
	 * This method should return true if the eventstore implementation supports local transactions.  
	 *
	 * @return 	true if local transactions are supported; false otherwise.
	 * @see 	com.ibm.j2ca.extension.eventmanagement.EventStore#isTransactional()
	 */
	public boolean isTransactional() {
        //TODO implement this
        return false;
	}
	
   /**
	 * Roll back the transaction
	 * 
	 * @throws 	CommException
	 * @throws 	ResourceException
	 * @see 	com.ibm.j2ca.extension.eventmanagement.EventStore#rollbackWork()
	 */
	public void rollbackWork() throws javax.resource.ResourceException,
			javax.resource.spi.CommException {	
        //TODO implement this
	}

	/**
	 * Commit the transaction
	 * 
	 * @throws 	CommException
	 * @throws 	ResourceException
	 * @see 	com.ibm.j2ca.extension.eventmanagement.EventStore#commitWork()
	 */
	public void commitWork() throws javax.resource.ResourceException,
			javax.resource.spi.CommException {
        //TODO implement this
	}

}